<?php
namespace App;

use Entity;

class Acl
{
    /** @var Entity\Repository\RolePermissionRepository */
    protected $permission_repo;

    /** @var array|null An array of actions enabled by each role. */
    protected $_actions = null;

    public function __construct(Entity\Repository\RolePermissionRepository $permission_repo)
    {
        $this->permission_repo = $permission_repo;
        $this->reload();
    }

    /**
     * Force a reload of the internal ACL cache (used in the event of a user status change.
     */
    public function reload(): void
    {
        $this->_actions = $this->permission_repo->getActionsForAllRoles();
        $this->_roles = null;
        $this->_cache = null;
    }

    /**
     * Check if a specified User entity is allowed to perform an action (or array of actions).
     *
     * @param Entity\User|null $user
     * @param $action
     * @return bool
     */
    public function userAllowed(?Entity\User $user = null, $action): bool
    {
        if (!($user instanceof Entity\User)) {
            return false;
        }

        $roles = [];
        if ($user->getRoles()->count() > 0) {
            foreach ($user->getRoles() as $role) {
                $roles[] = $role->getId();
            }
        }

        return $this->roleAllowed($roles, $action);
    }

    /**
     * Check if a role (or array of roles) is allowed to perform an action (or array of actions).
     *
     * @param int|array $role_id
     * @param string|array $action
     * @return bool
     */
    public function roleAllowed($role_id, $action)
    {
        // Iterate through an array of roles and return with the first "true" response, or "false" otherwise.
        if (\is_array($role_id)) {
            foreach ($role_id as $r) {
                if ($this->roleAllowed($r, $action)) {
                    return true;
                }
            }

            return false;
        }

        // If multiple actions are supplied, treat the list as "x OR y OR z", returning if any action is allowed.
        if (\is_array($action)) {
            foreach ($action as $a) {
                if ($this->roleAllowed($role_id, $a)) {
                    return true;
                }
            }

            return false;
        }

        if (\in_array('administer all', (array)$this->_actions[$role_id], true)) {
            return true;
        }

        if (isset($this->_actions[$role_id]) && \in_array($action, $this->_actions[$role_id], true)) {
            return true;
        }

        return false;
    }

    /**
     * Wrapper around the 'userAllowed' function that throws a UI-friendly exception upon failure.
     *
     * @param Entity\User|null $user
     * @param $action
     * @throws Exception\NotLoggedIn
     * @throws Exception\PermissionDenied
     */
    public function checkPermission(?Entity\User $user = null, $action)
    {
        if (!($user instanceof Entity\User)) {
            throw new \App\Exception\NotLoggedIn;
        }

        if (!$this->userAllowed($user, $action)) {
            throw new \App\Exception\PermissionDenied;
        }
    }
}